﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FastReg
{
	public partial class FormFastReg : Form
	{


		public static FormFastReg _FormFastReg;
		FastReg reg;
		Thread regThread;

		const String StatusBeginString = "Status: ";
		const String StatusProcessing = "processing...";
		const String StatusProcessingEnd = "End";

		bool NotifyIsEnabled = true;

		// Time:
		private DateTime startTime;
		private DateTime endTime;
		private TimeSpan processTime;


		public FormFastReg()
		{
			InitializeComponent();

			InitFastReg();
			
		}


		private void InitFastReg()
		{
			_FormFastReg = this;
			textBoxFastRegMarkCharacter.Text = "\t";
		}

		private void FormFastReg_FormClosing(object sender, FormClosingEventArgs e)
		{
			if(regThread!=null)
			{
				regThread.Abort();
			}

			// Notify close
			if(NotifyIsEnabled)
			{
				notifyIconApplication.Visible = false;
			}
		}



		/// <summary>
		/// Notify on taskbar
		/// </summary>
		/// <param name="message"></param>
		private void MessageForUser(String message)
		{
			if (NotifyIsEnabled)
			{
				notifyIconApplication.BalloonTipText = message;
				//notifyIconForParent.Text = message;   // Ikon neve
				notifyIconApplication.BalloonTipTitle = "FastReg";
				notifyIconApplication.ShowBalloonTip(1000);
			}
		}



		private void buttonFastRegProcess_Click(object sender, EventArgs e)
		{
			RunFastReg();

		}


		private void buttonFastRegProcessTest_Click(object sender, EventArgs e)
		{
			RunFastRegTest();
		}



		private void RunFastRegTest()
		{
			// Search "short" text

			String testString = "";

			if(richTextBoxFastRegInput.SelectedText.Length > 0)
			{
				// Has selected text
				testString = richTextBoxFastRegInput.Text.Substring(0, richTextBoxFastRegInput.SelectedText.Length);
			}
			else if (richTextBoxFastRegInput.Text.Length > 0)
			{
				// There is no selected text
				// Send first 100 character
				if (richTextBoxFastRegInput.Text.Length > 200)
				{
					// 100
					testString = richTextBoxFastRegInput.Text.Substring(0, 200);
				}
				else
				{
					//richTextBoxFastRegInput.Text.Length
					testString = richTextBoxFastRegInput.Text.Substring(0, richTextBoxFastRegInput.Text.Length);
				}
			}
			else
			{
				return;
			}

			// Run
			// TODO: new function !!
			StartProcessRegex(textBoxFastRegMarkCharacter.Text, testString, textBoxFastRegRegularExpression.Text.TrimEnd());

		}



		private void RunFastReg()
		{

			StartProcessRegex(textBoxFastRegMarkCharacter.Text, richTextBoxFastRegInput.Text, textBoxFastRegRegularExpression.Text.TrimEnd());

		}



		private void StartProcessRegex(string betweenMarkers, string inputString, string regexString)
		{
			startTime = DateTime.Now;
			String timeString = "Runtime: ";
			toolStripStatusLabelTime.Text = timeString;

			reg = new FastReg();

			toolStripStatusLabelFastReg.Text = StatusBeginString + StatusProcessing;

			reg.betweenMarkeds = betweenMarkers;
			reg.inputString = inputString;
			reg.regexString = regexString;

			//reg.SetProgressDelegate = (FastReg.SetProgressDelegateType)SetProgressBar;
			timerRefreshProgressBar.Enabled = true;
			timerRefreshProgressBar.Start();

			buttonFastRegProcess.Enabled = false;
			buttonFastRegProcessTest.Enabled = false;

			// Run
			// reg.Separator();
			reg.isRunning = true;
			reg.regFinishEvent = new ManualResetEvent(false);

			regThread = new Thread(reg.Process);
			regThread.Start();

			// Wait

			//regThread.Join();	// Wrong, blocking!


			/*
			while(regThread.Join(1000))
			{
				// Wait 1000 ms

				toolStripProgressBarFastReg.Value = reg.progressStatusPercent;
			}*/

			/*
			while (reg.isRunning)
			{
				Thread.Sleep(100);
			}*/

			//reg.regFinishEvent.WaitOne();


			//toolStripProgressBarFastReg.Value = reg.progressStatusPercent;
		}




		public void FastRegIsEnd()
		{
			richTextBoxFastRegOutput.Text = reg.outputString;

			timerRefreshProgressBar.Stop();

			buttonFastRegProcess.Enabled = true;
			buttonFastRegProcessTest.Enabled = true;

			toolStripStatusLabelFastReg.Text = StatusBeginString + StatusProcessingEnd;

			endTime = DateTime.Now;

			processTime = endTime - startTime;
			String timeString = "Runtime: ";
			timeString += processTime.Minutes + " min / ";
			timeString += processTime.Seconds + " second";
			toolStripStatusLabelTime.Text = timeString;

			MessageForUser("End processing...");
		}



		public void SetProgressBar(int value)
		{
			// TODO: delete
			if (InvokeRequired)
			{
				// Old version, there is no try-catch{}
				//this.Invoke(new Action<string>(SetProgressBar), new object[] { value });
				this.Invoke(new Action<int>(SetProgressBar), new object[] { value });
			}

			toolStripProgressBarFastReg.Value = value;

		}



		public void SetProgressBar()
		{
			if (reg != null)
			{
				toolStripProgressBarFastReg.Value = reg.progressStatusPercent; 
			}
		}



		private void timerRefreshProgressBar_Tick(object sender, EventArgs e)
		{
			SetProgressBar();

			timerRefreshProgressBar.Start();

			if(!reg.isRunning)
			{
				FastRegIsEnd();
			}
		}

		private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			MessageBox.Show("Programmed by Vizi Gábor\n" +
							"www.fasten.e5tv.hu\n","About");
		}
	}
}
